## ----setup, include=FALSE, error=FALSE----------------------------------------
# withSimplace <- FALSE
# try({
#   withSimplace <- !is.na(simplace::findFirstSimplaceInstallation())
# })

knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(message = FALSE)
knitr::opts_chunk$set(warning = FALSE)

includeGraphic <- function(gr,name) {
  
    type <- ifelse(knitr::is_latex_output(),'pdf','png')
    fn <- knitr::fig_path(type,number=as.integer(runif(1,0,10000)))
    dir.create(dirname(fn),recursive = TRUE, showWarnings = FALSE)
    DiagrammeR::export_graph(gr, fn, width = 2000)
    if(!file.exists(fn))
    {
      fn <- system.file("img",paste0(name,'.png'),package="simplaceUtil")
    }
    try(knitr::include_graphics(fn), silent=TRUE)
}

rawOutput <- function(txt,type="") {
  cat(paste0("```",type,"\n"))
  cat(paste0(trimws(txt),collapse="\n"))
  cat("\n```")
}




## -----------------------------------------------------------------------------
library(simplaceUtil)

## -----------------------------------------------------------------------------
solfile <- system.file("solution","Complete.sol.xml",
                       package="simplaceUtil")

sol <- getSolutionFromFile(solfile)
components <- getComponents(sol)
variables <- getUserVariables(sol)
links <- getLinks(sol, components)

## -----------------------------------------------------------------------------
el <- getElementsFromSolutionFile(solfile)
sol <- el$solution
components <- el$components
variables <- el$variables
links <- el$links

## ---- echo=FALSE--------------------------------------------------------------
components$ref <- gsub('$', '\\$', components$ref, fixed = TRUE)

## -----------------------------------------------------------------------------
components[c(1:3,30:31,42:44),c("id","type","subtype","nr")]

## -----------------------------------------------------------------------------
links[c(1:2,23,300,313,322,325),]

## -----------------------------------------------------------------------------
variables[c(1:5,18:20),]

## -----------------------------------------------------------------------------
head(components[,c("id","ref")])

## ---- echo=FALSE--------------------------------------------------------------
components$ref <- gsub('\\$', '$', components$ref, fixed = TRUE)

## -----------------------------------------------------------------------------
components$ref <- replaceVariablesWithValues(
  components$ref,
  variables, 
  additional=c("_WORKDIR_"="~"))
head(components[,c("id","ref")])

## -----------------------------------------------------------------------------
ncomp <- components[components$type=="simcomponent" & components$subtype=="normal",]
ncomp$id

## -----------------------------------------------------------------------------
nlinks <- links[links$from %in% ncomp$id & links$to %in% ncomp$id,]
head(nlinks)

## -----------------------------------------------------------------------------
nlinks <- aggregate(name ~ from + to , data=nlinks, length)
nlinks <- nlinks[order(-nlinks$name),]
head(nlinks,10)

## -----------------------------------------------------------------------------
wd <- system.file(package="simplaceUtil")
solfiles <- list.files(wd,pattern=".sol.xml",recursive=TRUE,full.names = TRUE)

df <- do.call(rbind,
        lapply(solfiles,getSolutionInfoAsDataframe,workdir=wd))
df$file <- basename(df$file)
df[df$type=="simcomponent",c("id","file")] 

## -----------------------------------------------------------------------------
graph <- solutionToGraph(solfile)

## ----eval=FALSE---------------------------------------------------------------
#  DiagrammeR::render_graph(graph)

## ----echo=FALSE, out.width="100%"---------------------------------------------
includeGraphic(graph,"graph")

## -----------------------------------------------------------------------------
subgraph <- getNeighborhood(graph, c("SlimWater","SlimRoots"),
                            distance=1, type="simcomponent")

## ----eval=FALSE---------------------------------------------------------------
#  DiagrammeR::render_graph(subgraph)

## ----echo=FALSE, out.width="100%"---------------------------------------------
includeGraphic(subgraph,"subgraph")

## -----------------------------------------------------------------------------
solfile <- system.file("solution","Yield.sol.xml",
                       package="simplaceUtil")
sol <- getSolutionFromFile(solfile)
newsol <- sol |>
  removeNonMemoryOutputs() |>
  addMemoryOutput("Yields",frequence = "YEARLY") |>
  addOutputVariable("Yields","Year","CURRENT.YEAR","INT") |>
  addOutputVariable("Yields", "ClimateZone","vClimateZone","CHAR") |>
  addOutputVariable("Yields", "Yield","LintulBiomass.sWSO","DOUBLE","MAX") |>
  addUserVariable("vLUE",3.0,"DOUBLE") |>
  addUserVariable("vSLA",0.02,"DOUBLE") |>
  addUserVariable("vRGRL",0.009,"DOUBLE") |>
  replaceVariable("crop.LUE","vLUE") |>
  replaceVariable("crop.SLA","vSLA") |>
  replaceVariable("crop.RGRL","vRGRL")


## -----------------------------------------------------------------------------
soltext <- getTextFromSolution(newsol)

## ----echo=FALSE, results='asis'-----------------------------------------------
rawOutput(substr(soltext,1,1530))


## ----eval=FALSE---------------------------------------------------------------
#  solution_modified <- "simulation/solution/YieldModified.sol.xml"
#  writeSolutionToFile(newsol, solution_modified)

## -----------------------------------------------------------------------------
solfile <- system.file("solution","Yield.sol.xml",
                       package="simplaceUtil")
sol <- getSolutionFromFile(solfile)

cmp <- getComponents(sol)
cmp[cmp$type=="simcomponent",c("id","subtype","nr")]

## ----echo=FALSE---------------------------------------------------------------
set.seed(1234)

## -----------------------------------------------------------------------------
toswap <- 2:5
swapped <- sample(toswap, length(toswap))
swapped
newsol <- swapComponents(sol,swapped)
cmp <- getComponents(newsol)
cmp[cmp$type=="simcomponent",c("id","subtype","nr")]

## -----------------------------------------------------------------------------
solfile <- system.file("solution","Yield.sol.xml",
                       package="simplaceUtil")
sol <- getSolutionFromFile(solfile)

newsol <- addTimingSimComponent(sol, componentlist=c("LintulBiomass","LintulPheno"))
cmp <- getComponents(newsol)
cmp[cmp$type=="simcomponent" | grepl("timing",cmp$id),c("id","type","subtype","nr")]


## -----------------------------------------------------------------------------
outfile <- system.file("output","water.csv",
                       package="simplaceUtil")
water <- read.csv(outfile, header=TRUE, sep=";")
water <- parseDate(water)

## -----------------------------------------------------------------------------
water[1:4,c(2,9:11,49)]
water_long <- transformLayeredData(water)
water_long[c(1:3,41:43,81:83),c(2,9:11)]

## ----fig.width=7, fig.height=5------------------------------------------------
plotScalarOutput(water,"CURRENT.DATE",c("Evaporation","Transpiration"))

## ----fig.width=7, fig.height=5------------------------------------------------
plotLayeredOutput(water,"RetainedWater")

## ----eval=FALSE---------------------------------------------------------------
#  runSimplaceGuiApp()

## ----out.width="100%", echo=FALSE---------------------------------------------
fn <- system.file("img","shiny_graph.png",package="simplaceUtil")
try(knitr::include_graphics(fn), silent=TRUE)

## ----out.width="100%", echo=FALSE---------------------------------------------
fn <- system.file("img","shiny_plot.png",package="simplaceUtil")
try(knitr::include_graphics(fn), silent=TRUE)

## -----------------------------------------------------------------------------
stubs <- createCsvResourceStubs(
  filename = system.file("input","weather.csv", package="simplaceUtil"),
  id = "weather",
  sep =",",
  keyvals = c("CURRENT.DATE" = "Date")
)
cat(stubs$interface)
cat(stubs$resource)

## -----------------------------------------------------------------------------
varfile <- system.file("variables","translocation.csv",package="simplaceUtil")
vartable <- read.csv(varfile, header=TRUE, sep=";")
vartable[c(4,8:9),]

## -----------------------------------------------------------------------------
stubs <- createCodeStubsForSimVariables(varfile, "BMTransloc", sep=";")
names(stubs)

## ----eval=FALSE---------------------------------------------------------------
#  stubs <- createCodeStubsForSimVariables(varfile, "BMTransloc", sep=";",
#                                          outfolder="translocation_stubs")

## ----echo=FALSE, results='asis'-----------------------------------------------
rawOutput(stubs$JavaFields[c(4,6,14)])

## ----echo=FALSE, results='asis'-----------------------------------------------
rawOutput(stubs$JavaVariables[c(4,6,14)])

## ----echo=FALSE, results='asis'-----------------------------------------------
rawOutput(stubs$JavaInit[c(6,14)])

## ----echo=FALSE, results='asis'-----------------------------------------------
rawOutput(stubs$ComponentInputsXML[c(3:5)])

## ----echo=FALSE, results='asis'-----------------------------------------------
rawOutput(stubs$ParametersXML)

## ----echo=FALSE, results='asis'-----------------------------------------------
rawOutput(stubs$ResourceParametersXML)

